# -*- coding: utf-8 -*-
# Generated by Django 1.11.29 on 2021-01-26 14:47
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('djelmah', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='djelmahhost',
            name='api_key',
            field=models.CharField(blank=True, max_length=128),
        ),
        migrations.AlterField(
            model_name='djelmahhost',
            name='username',
            field=models.CharField(blank=True, max_length=150),
        ),
    ]



from django.contrib import admin
from djelmah import models

@admin.register(models.DjelmahHost)
class DjelmahHostAdmin(admin.ModelAdmin):
    list_display = (
        'hostname',
        'username',
        'api_key',
        'active'
    )

@admin.register(models.DjelmahLog)
class DjelmahLogAdmin(admin.ModelAdmin):
    list_display = (
        'host',
        'path',
        'username',
        'datetime',
        'error_type',
        'status_code'
    )



class DjelmahLogMiddleware:
    def __init__(self, get_response):
        self.get_response = get_response
    
    def __call__(self, request):
        response = self.get_response(request)
        return response

    def process_exception(self, request, exception):
        """log unhandled 500 errors"""
        import json, requests, sys
        from datetime import datetime
        from django.conf import settings
        from django.forms.models import model_to_dict
        from django.views.debug import ExceptionReporter
        from models import DjelmahHost, DjelmahLog

        reporter = ExceptionReporter(
            request,
            exception.__class__,
            exception.message,
            sys.exc_info()[2]
        )

        log = DjelmahLog(
            host=request.get_host(),
            path=request.get_full_path(),
            username=request.user.username,
            datetime=datetime.now(),
            error_type=exception.__class__.__name__,
            error_message=exception.message,
            status_code="500",
            raw_html=reporter.get_traceback_html()
        )
        
        data = json.dumps(
            model_to_dict(log), default=lambda obj: obj.__str__()
        )

        headers = { 'Content-Type': "application/json" }
        hosts = DjelmahHost.objects.filter(active=True)

        for host in hosts:
            if host.hostname == "localhost":
                log.save()
                
            else:
                headers['Authorization'] = "ApiKey {}:{}".format(
                    host.username, host.api_key
                )

                url = host.hostname + "/djelmah/api/v1/logs/"
                requests.post(url, data=data, headers=headers)
            
        return None



# -*- coding: utf-8 -*-
# Generated by Django 1.11.15 on 2018-11-11 15:55
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='DjelmahHost',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('hostname', models.CharField(max_length=256)),
                ('username', models.CharField(max_length=150)),
                ('api_key', models.CharField(max_length=128)),
                ('active', models.BooleanField(default=True)),
            ],
        ),
        migrations.CreateModel(
            name='DjelmahLog',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('host', models.CharField(max_length=128)),
                ('path', models.CharField(max_length=2048)),
                ('username', models.CharField(max_length=32)),
                ('datetime', models.DateTimeField()),
                ('error_type', models.CharField(max_length=64)),
                ('error_message', models.TextField(blank=True)),
                ('status_code', models.CharField(blank=True, max_length=8)),
                ('raw_html', models.TextField(blank=True)),
            ],
        ),
    ]



from datetime import datetime, timedelta
from django.contrib.auth.models import User
from django.http import HttpResponse, HttpResponseRedirect
from django.shortcuts import get_object_or_404, render
from django.views.decorators.http import require_GET, require_POST
from tastypie.models import ApiKey
from models import DjelmahLog

# LOGS

@require_GET
def index(request):
    return HttpResponseRedirect('/djelmah/logs')

@require_GET
def log_index(request):
    hosts = DjelmahLog.objects.values_list('host', flat=True).distinct()
    logs = DjelmahLog.objects.all().order_by('-datetime')

    if request.GET:
        selected_host = request.GET.get('host', None)

        start_date = request.GET.get('start_date', None)
        start_date = datetime.strptime(start_date, '%Y-%m-%d') if start_date else None
    
        end_date = request.GET.get('end_date', None)
        end_date = datetime.strptime(end_date, '%Y-%m-%d') if end_date else None
    else:
        selected_host = None
        end_date = datetime.today()
        start_date = end_date - timedelta(days=30)

    if selected_host:
        logs = logs.filter(host=selected_host)

    if start_date:
        logs = logs.filter(datetime__gte=start_date)

    if end_date:
        logs = logs.filter(datetime__lte=end_date + timedelta(days=1))

    context = {
        'selected_host': selected_host,
        'start_date': start_date,
        'end_date': end_date,
        'hosts': hosts,
        'logs': logs,
    }

    return render(request, 'djelmah/logs.html', context)

@require_GET
def log_detail(request, log_id):
    log = get_object_or_404(DjelmahLog, pk=log_id)
    return HttpResponse(log.raw_html)

@require_POST
def delete_log(request):
    log_id = request.POST['log_id']
    log = get_object_or_404(DjelmahLog, pk=log_id)

    log.delete()

    return HttpResponse()

@require_GET
def log_test(request):
    raise Exception("It Works!")

# API KEYS

@require_GET
def api_keys(request):
    keys = ApiKey.objects.all().order_by('user__username')
    users = User.objects.exclude(
        id__in=keys.values_list('user_id', flat=True)
    ).order_by('last_name', 'first_name')

    context = {
        'keys': keys,
        'users': users,
    }

    return render(request, 'djelmah/keys.html', context)

@require_POST
def create_api_key(request):
    user_id = request.POST['user_id']
    user = get_object_or_404(User, pk=user_id)

    key, created = ApiKey.objects.update_or_create(
        user=user, defaults={ 'key': None }
    )

    return HttpResponse()

@require_POST
def revoke_api_key(request):
    key_id = request.POST['key_id']
    key = get_object_or_404(ApiKey, pk=key_id)

    key.delete()

    return HttpResponse()



from django.contrib.auth import authenticate, login, logout
from django.contrib.auth.decorators import login_required
from django.http import HttpResponseRedirect
from django.shortcuts import render

def auth_login(request):
    if request.POST:
        user = authenticate(
            username=request.POST['username'],
            password=request.POST['password']
        )

        if user is not None and user.is_active:
            login(request, user)
            
        return HttpResponseRedirect('/')
        
    return render(request, 'djelmah/login.html')

@login_required
def auth_logout(request):
    logout(request)
    return HttpResponseRedirect('/login')

@login_required
def index(request):
    return HttpResponseRedirect('/djelmah')



from datetime import datetime
from tastypie import fields
from tastypie.authentication import ApiKeyAuthentication
from tastypie.authorization import Authorization
from tastypie.resources import ModelResource
from models import DjelmahLog

class DjelmahLogResource(ModelResource):
    class Meta:
        resource_name = "logs"
        max_limit = 0
        limit = 0

        queryset = DjelmahLog.objects.all()
        filtering = {
            'host': {'exact'}
        }

        authentication = ApiKeyAuthentication()
        authorization = Authorization()



from django.conf.urls import url
from . import views

urlpatterns = [
    url(r'^$', views.index),
    url(r'^logs/$', views.log_index),
    url(r'^logs/(?P<log_id>\d+)$', views.log_detail),
    url(r'^logs/delete$', views.delete_log),
    url(r'^logs/test$', views.log_test),
    url(r'^keys/$', views.api_keys),
    url(r'^keys/create$', views.create_api_key),
    url(r'^keys/revoke$', views.revoke_api_key),
]



import os
from setuptools import find_packages, setup

with open(os.path.join(os.path.dirname(__file__), 'README.md')) as readme:
    README = readme.read()

# allow setup.py to be run from any path
os.chdir(os.path.normpath(os.path.join(os.path.abspath(__file__), os.pardir)))

setup(
    name='django-elmah',
    version='1.1',
    packages=find_packages(exclude=['djelmah-server']),
    include_package_data=True,
    license='GNU Affero General Public License v3',
    description='Django Error Logging Modules and Handlers',
    long_description=README,
    url='https://github.com/rickjordan/django-elmah',
    author='Rick Jordan',
    author_email='jordan.rick.d@gmail.com',
    install_requires=[
        'django-tastypie',
        'requests',
    ],
    classifiers=[
        'Environment :: Web Environment',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: GNU Affero General Public License v3',
        'Natural Language :: English',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
    ],
)



from django.conf.urls import include, url
from tastypie.api import Api
from . import resources

# api resources
api = Api(api_name='v1')
api.register(resources.DjelmahLogResource())

urlpatterns = [
    url(r'^', include(api.urls)),
]



from django.db import models

class DjelmahHost(models.Model):
    hostname = models.CharField(max_length=256)
    username = models.CharField(max_length=150, blank=True)
    api_key = models.CharField(max_length=128, blank=True)
    active = models.BooleanField(default=True)

class DjelmahLog(models.Model):
    host = models.CharField(max_length=128)
    path = models.CharField(max_length=2048)
    username = models.CharField(max_length=32)
    datetime = models.DateTimeField()
    error_type = models.CharField(max_length=64)
    error_message = models.TextField(blank=True)
    status_code = models.CharField(max_length=8, blank=True)
    raw_html = models.TextField(blank=True)
